// Scintilla source code edit control
// CallTip.cxx - code for displaying call tips
// Copyright 1998-1999 by Neil Hodgson <neilh@hare.net.au>
// The License.txt file describes the conditions under which this software may be distributed.

#include <stdlib.h>
#include <string.h>

#include "Platform.h"

#include "CallTip.h"

CallTip::CallTip() {
	wCallTip = 0;
	inCallTipMode = false;
	posStartCallTip = 0;
	val = 0;
	startHighlight = 0;
	endHighlight = 0;

	colourBG.desired = Colour(0xff, 0xff, 0xff);
	colourUnSel.desired = Colour(0x80, 0x80, 0x80);
	colourSel.desired = Colour(0, 0, 0x80);
	colourShade.desired = Colour(0, 0, 0);
	colourLight.desired = Colour(0xc0, 0xc0, 0xc0);
}

CallTip::~CallTip() {
	wCallTip.Destroy();
	if (val) 
		free(val);
	val = 0;
}

void CallTip::RefreshColourPalette(Palette &pal, bool want) {
	pal.WantFind(colourBG, want);
	pal.WantFind(colourUnSel, want);
	pal.WantFind(colourSel, want);
	pal.WantFind(colourShade, want);
	pal.WantFind(colourLight, want);
}

void CallTip::PaintCT(Surface *surfaceWindow) {
	if (!val)
		return;
	PRectangle rcClientPos = wCallTip.GetClientPosition();
	PRectangle rcClientSize(0,0,rcClientPos.right - rcClientPos.left, 
		rcClientPos.bottom - rcClientPos.top);
	PRectangle rcClient(1,1,rcClientSize.right - 1, rcClientSize.bottom - 1);
	
	surfaceWindow->FillRectangle(rcClient, colourBG.allocated);
	// To make a nice small call tip window, it is only sized to fit most normal characters without accents
	int ascent = surfaceWindow->Ascent(font) - surfaceWindow->InternalLeading(font);

	// Draw the definition in three parts: before highlight, highlighted, after highlight
	int x = 5;
	int xEnd = x + surfaceWindow->WidthText(font, val, startHighlight);
	int ytext = rcClient.top + ascent + 1;
	rcClient.left = x;
	rcClient.right = xEnd;
	surfaceWindow->DrawText(rcClient, font, ytext, 
		val, startHighlight, 
		colourUnSel.allocated, colourBG.allocated);
	x = xEnd;

	xEnd = x + surfaceWindow->WidthText(font, val + startHighlight, 
		endHighlight - startHighlight);
	rcClient.left = x;
	rcClient.right = xEnd;
	surfaceWindow->DrawText(rcClient, font, ytext, 
		val + startHighlight, endHighlight - startHighlight, 
		colourSel.allocated, colourBG.allocated);
	x = xEnd;

	xEnd = x + surfaceWindow->WidthText(font, val + endHighlight, 
		strlen(val) - endHighlight);
	rcClient.left = x;
	rcClient.right = xEnd;
	surfaceWindow->DrawText(rcClient, font, ytext, 
		val + endHighlight, strlen(val) - endHighlight, 
		colourUnSel.allocated, colourBG.allocated);
	x = xEnd;

	// Draw a raised border around the edges of the window
	surfaceWindow->MoveTo(0, rcClientSize.bottom - 1);
	surfaceWindow->PenColour(colourShade.allocated);
	surfaceWindow->LineTo(rcClientSize.right - 1, rcClientSize.bottom - 1);
	surfaceWindow->LineTo(rcClientSize.right - 1, 0);
	surfaceWindow->PenColour(colourLight.allocated);
	surfaceWindow->LineTo(0, 0);
	surfaceWindow->LineTo(0, rcClientSize.bottom - 1);
}

PRectangle CallTip::CallTipStart(int pos, Point pt, char *defn, 
	const char *faceName, int size) {
	Surface surfaceMeasure;
	surfaceMeasure.Init();
	int deviceHeight = (size * surfaceMeasure.LogPixelsY()) / 72;
	font.Create(faceName, deviceHeight);
	if (val)
		free(val);
	val = strdup(defn);
	if (!val)
		return PRectangle();
	startHighlight = 0;
	endHighlight = 0;
	inCallTipMode = true;
	posStartCallTip = pos;
	int width = surfaceMeasure.WidthText(font, val, strlen(val)) + 10;
	int lineHeight = surfaceMeasure.Height(font);
	// Extra line for border and an empty line at top and bottom
	int height = lineHeight - surfaceMeasure.InternalLeading(font) + 4;
	return PRectangle(pt.x-5, pt.y + lineHeight + 2, pt.x + width - 5, pt.y + lineHeight + 2 + height);
}

void CallTip::CallTipCancel() {
	inCallTipMode = false;
	if (wCallTip.Created()) {
		wCallTip.Destroy();
	}
}

void CallTip::SetHighlight(int start, int end) {
	// Avoid flashing by checking something has really changed
	if ((start != startHighlight) || (end != endHighlight)) {
		startHighlight = start;
		endHighlight = end;
		if (wCallTip.Created()) {
			wCallTip.InvalidateAll();
		}
	}
}
