// Scintilla source code edit control
// ViewStyle.cxx - store information on how the document is to be viewed
// Copyright 1998-1999 by Neil Hodgson <neilh@hare.net.au>
// The License.txt file describes the conditions under which this software may be distributed.

#include <string.h>

#include "Platform.h"

#include "Scintilla.h"
#include "Indicator.h"
#include "LineMarker.h"
#include "Style.h"
#include "ViewStyle.h"

ViewStyle::ViewStyle() {
	indicators[0].style = INDIC_SQUIGGLE;
	indicators[0].fore = Colour(0, 0x7f, 0);
	indicators[1].style = INDIC_TT;
	indicators[1].fore = Colour(0, 0, 0xff);
	indicators[2].style = INDIC_PLAIN;
	indicators[2].fore = Colour(0xff, 0, 0);

	lineHeight = 1;
	maxAscent = 1;
	maxDescent = 1;
	aveCharWidth = 8;
	spaceWidth = 8;
	foreground.desired = Colour(0, 0, 0);
	background.desired = Colour(0xff, 0xff, 0xff);

	size = Platform::DefaultFontSize();
	strcpy(fontName, Platform::DefaultFont());
	bold = false;
	italic = false;

	selforeset = false;
	selforeground.desired = Colour(0xff, 0, 0);
	selbackset = true;
	selbackground.desired = Colour(0xc0, 0xc0, 0xc0);
	selbar.desired = Platform::Chrome();
	selbarlight.desired = Platform::ChromeHighlight();
	// TODO: the line numbers should have their own style
	linenumfore.desired = Colour(0, 0, 0);
	//caretcolour.desired = Colour(0xff, 0, 0);
	caretcolour.desired = Colour(0, 0, 0);
	
	selMarginWidth = 20;
	lineNumberWidth = 0;
	fixedColumnWidth = selMarginWidth + lineNumberWidth;
	zoomLevel = 0;
	viewWhitespace = false;
}

ViewStyle::~ViewStyle() {
}

void ViewStyle::RefreshColourPalette(Palette &pal, bool want) {
	unsigned int i;
	for (i=0;i<(sizeof(styles)/sizeof(styles[0]));i++) {
		pal.WantFind(styles[i].fore, want);
		pal.WantFind(styles[i].back, want);
	}
	for (i=0;i<(sizeof(indicators)/sizeof(indicators[0]));i++) {
		pal.WantFind(indicators[i].fore, want);
	}
	for (i=0;i<(sizeof(markers)/sizeof(markers[0]));i++) {
		pal.WantFind(markers[i].fore, want);
		pal.WantFind(markers[i].back, want);
	}
	pal.WantFind(foreground, want);
	pal.WantFind(background, want);
	pal.WantFind(selforeground, want);
	pal.WantFind(selbackground, want);
	pal.WantFind(selbar, want);
	pal.WantFind(selbarlight, want);
	pal.WantFind(linenumfore, want);
	pal.WantFind(caretcolour, want);
}

void ViewStyle::Refresh() {
	selbar.desired = Platform::Chrome();
	selbarlight.desired = Platform::ChromeHighlight();
	maxAscent = 1;
	maxDescent = 1;
	for (unsigned int i=0;i<(sizeof(styles)/sizeof(styles[0]));i++) {
		styles[i].Realise(zoomLevel);
		if (maxAscent < styles[i].ascent)
			maxAscent = styles[i].ascent;
		if (maxDescent < styles[i].descent)
			maxDescent = styles[i].descent;
	}
	
	lineHeight = maxAscent + maxDescent;
	aveCharWidth = styles[0].aveCharWidth;
	spaceWidth = styles[0].spaceWidth;

	fixedColumnWidth = selMarginWidth + lineNumberWidth;
}

void ViewStyle::ClearStyles() {
	for (int i=0; i<=STYLE_MAX; i++) {
		styles[i].Clear(foreground.desired, background.desired, 
			size, fontName, bold, italic);
	}
}

